package models.scrum

import org.eclipse.emf.ecore.EObject
import uk.ac.kcl.interpreter.IGuidanceFunction
import org.eclipse.emf.common.util.EList
import org.apache.commons.math3.stat.descriptive.moment.StandardDeviation

class MinimiseSprintEffortDeviation implements IGuidanceFunction {
	
	override computeFitness(EObject model) {
				
		var effortPerSprint = (model.getFeature("sprints") as EList<EObject>).map[ sprint | 
			
			new Double((sprint.getFeature("committedItem") as EList<EObject>).fold(0d)[ result, item |
				
				result + item.getFeature("Effort") as Integer
				
			])
			
		].toList
		
		var effortStandardDeviation = new StandardDeviation().evaluate(effortPerSprint)
		
		println("Sprint effort distribution: " + effortPerSprint)
		println("Sprint effort standard deviation: " + effortStandardDeviation)
		
		return effortStandardDeviation
	}
	
	override getName() {
		return "Minimise sprint effort deviation"
	}

	/**
	 * Helper function getting the value of the named feature (if it exists) for the given EObject.
	 */
	def Object getFeature (EObject o, String feature) {
		
		if(o === null){
			println("Null object given")
		}
		
		o.eGet(o.eClass.getEStructuralFeature(feature))
		
	}	
}
